@@ -5,21 +5,159 @@ use strict;
 use warnings;
 use 5.12.0;
 
-sub import {
-  my $class = shift;
+our $VERSION = '0.30'; # VERSION
 
-  @_ = qw(Apex Constants Manifest Metadata Partner SessionManager Tooling Zip)
-    unless @_;
+use Data::Dumper;
+use Log::Log4perl ':easy';
 
-  for (@_) {
-    my $module = "WWW/SFDC/$_.pm";
-    require $module;
+use Moo;
+
+has 'apiVersion',
+  is => 'ro',
+  isa => sub { LOGDIE "The API version ($_[0]) must be >= 31." unless $_[0] and $_[0] >= 31},
+  default => '33.0';
+
+has 'loginResult',
+  is => 'rw',
+  lazy => 1,
+  builder => '_login';
+
+has 'password',
+  is => 'ro',
+  required => 1;
+
+has 'pollInterval',
+  is => 'rw',
+  default => 15;
+
+has 'attempts',
+  is => 'rw',
+  default => 3;
+
+has 'url',
+  is => 'ro',
+  default => "https://test.salesforce.com",
+  isa => sub { $_[0] and $_[0] =~ s/\/$// or 1; }; #remove trailing slash
+
+has 'username',
+  is => 'ro',
+  required => 1;
+
+INIT: {
+  for my $module (qw'
+    Apex Constants Metadata Partner Tooling
+  '){
+    has $module,
+      is => 'ro',
+      lazy => 1,
+      default => sub {
+        my $self = shift;
+        require "WWW/SFDC/$module.pm"; ## no critic
+        "WWW::SFDC::$module"->new(session => $self);
+      };
+  }
+}
+
+sub _login {
+  my $self = shift;
+
+  INFO "Logging in...\t";
+
+  my $request = SOAP::Lite
+    ->proxy(
+      $self->url()."/services/Soap/u/".$self->apiVersion()
+    )
+    ->readable(1)
+    ->ns("urn:partner.soap.sforce.com","urn")
+    ->call(
+      'login',
+      SOAP::Data->name("username")->value($self->username),
+      SOAP::Data->name("password")->value($self->password)
+    );
+
+  TRACE "Request: " . Dumper $request;
+  WWW::SFDC::CallException->throw(
+    message => "Login failed: " . $request->faultstring,
+    request => $request
+  ) if $request->fault;
+  return $request->result();
+}
+
+
+sub _doCall {
+  my $self = shift;
+  my ($attempts, $URL, $NS, @stuff) = @_;
+
+  INFO "Starting $stuff[0] request";
+  if (
+    my $result = eval {
+      SOAP::Lite
+        ->proxy($URL)
+        ->readable(1)
+        ->default_ns($NS)
+        ->call(
+          @stuff,
+          SOAP::Header->name("SessionHeader" => {
+            "sessionId" => $self->loginResult()->{"sessionId"}
+          })->uri($NS)
+        )
+    }
+  ) {
+    return $result;
+
+  } elsif ($attempts--) {
+    INFO "$stuff[0] failed: $@";
+    INFO "Retrying ($attempts attempts remaining)";
+    return $self->_doCall($attempts, $URL, $NS, @stuff);
+
+  } else {
+    WWW::SFDC::CallException->throw(
+      message => "$stuff[0] failed: " . $@
+    );
+  }
+}
+
+sub call {
+  my $self = shift;
+  my $req;
+
+  while (
+    $req = $self->_doCall($self->attempts, @_)
+    and $req->fault
+  ) {
+    TRACE "Operation request " => Dumper $req;
+    if ($req->faultstring =~ /INVALID_SESSION_ID/) {
+      $self->loginResult($self->_login());
+    } else {
+      WWW::SFDC::CallException->throw(
+        message => "$_[2] failed: " . $req->faultstring,
+        request => $req
+      );
+    }
   }
 
+  return $req;
+};
+
+
+sub isSandbox {
+  my $self = shift;
+  return $self->loginResult->{sandbox} eq  "true";
 }
 
 1;
 
+package WWW::SFDC::CallException;
+use strict;
+use warnings;
+use Moo;
+with 'WWW::SFDC::Role::Exception';
+
+has 'request',
+  is => 'ro';
+
+1;
+
 __END__
 
 =pod
@@ -30,7 +168,7 @@ WWW::SFDC - Wrappers around the Salesforce.com APIs.
 
 =head1 VERSION
 
-version 0.25
+version 0.30
 
 =head1 SYNOPSIS
 
@@ -45,6 +183,15 @@ for the construction of powerful and flexible deployment tools.
     use WWW::SFDC qw'Metadata Manifest';
         # Equivalent to importing WWW::SFDC::Metadata and WWW::SFDC::Manifest
 
+=head1 METHODS
+
+=head2 call
+
+=head2 isSandbox
+
+Returns 1 if the org associated with the given credentials are a sandbox. Use to
+decide whether to sanitise metadata or similar.
+
 =head1 CONTENTS
 
 =over 4
