@@ -5,19 +5,20 @@ use 5.12.0;
 use strict;
 use warnings;
 
+our $VERSION = '0.30'; # VERSION
+
 use XML::Parser;
 use Scalar::Util qw(blessed);
 use List::Util 1.29 qw'first reduce pairmap pairgrep pairfirst';
 use Data::Dumper;
 use Log::Log4perl ':easy';
-use WWW::SFDC::Constants qw(needsMetaFile hasFolders getEnding getDiskName getName);
 
 use Moo;
 
+has 'constants', is => 'ro', required => 1;
 has 'manifest', is => 'rw', default => sub { {} };
 has 'isDeletion', is => 'ro';
-has 'srcDir', is => 'rw', default => 'src';
-has 'apiVersion', is => 'rw', default => 33;
+has 'apiVersion', is => 'rw', default => '34';
 
 
 # _splitLine($line)
@@ -42,7 +43,9 @@ sub _splitLine {
   $result{"folder"} = $1
     if $line =~ /\/(\w+)\//;
 
-  my $extension = getEnding($result{"type"});
+  my $extension = (grep {$_ eq $result{type}} keys $self->constants->TYPES)
+    ? $self->constants->getEnding($result{"type"})
+    : undef;
 
   if ($line =~ /\/(\w+)-meta.xml/) {
     $result{"name"} = $1
@@ -82,11 +85,11 @@ sub _getFilesForLine {
     ?(
       "$split{folder}-meta.xml",
       "$split{folder}/$split{name}$split{extension}",
-      (needsMetaFile($split{"type"}) ? "$split{folder}/$split{name}$split{extension}-meta.xml" : ())
+      ($self->constants->needsMetaFile($split{"type"}) ? "$split{folder}/$split{name}$split{extension}-meta.xml" : ())
      )
     :(
       "$split{name}$split{extension}",
-      (needsMetaFile($split{"type"}) ? "$split{name}$split{extension}-meta.xml" : ())
+      ($self->constants->needsMetaFile($split{"type"}) ? "$split{name}$split{extension}-meta.xml" : ())
      )
    )
 }
@@ -113,14 +116,14 @@ sub getFileList {
   my $self = shift;
 
   return map {
-    my $type = getDiskName($_);
-    my $ending = getEnding($type) || "";
+    my $type = $self->constants->getDiskName($_);
+    my $ending = $self->constants->getEnding($type) || "";
 
     map {
-      if (hasFolders($type) and $_ !~ /\//) {
+      if ($self->constants->hasFolders($type) and $_ !~ /\//) {
         "$type/$_-meta.xml";
       } else {
-	      "$type/$_$ending", (needsMetaFile($type) ? "$type/$_$ending-meta.xml" : () );
+	      "$type/$_$ending", ($self->constants->needsMetaFile($type) ? "$type/$_$ending-meta.xml" : () );
       }
     } @{ $self->manifest->{$_} }
   } keys %{$self->manifest};
@@ -144,9 +147,9 @@ sub addList {
   my $self = shift;
 
   return reduce {$a->add($b)} $self, map {
-    DEBUG "Adding $_ to manifest";
+    TRACE "Adding $_ to manifest";
     +{
-      getName($$_{type}) => [
+      $self->constants->getName($$_{type}) => [
         defined $$_{folder}
           ? (
               ($self->isDeletion ? () : $$_{folder}),
@@ -214,22 +217,22 @@ WWW::SFDC::Manifest - Utility functions for Salesforce Metadata API interactions
 
 =head1 VERSION
 
-version 0.25
+version 0.30
 
 =head1 SYNOPSIS
 
 This module is used to read SFDC manifests from disk, add files to them,
 and get a structure suitable for passing into WWW::SFDC::Metadata functions.
 
-   my $Manifest = WWW::SFDC::Manifest
-        ->new()
+    my $SFDC = WWW::SFDC->new(...);
+    my $Manifest = WWW::SFDC::Manifest
+        ->new(constants => $SFDC->Constants)
         ->readFromFile("filename")
-        ->add(
-            WWW::SFDC::Manifest->new()->readFromFile("anotherFile")
-        )->add({Document => ["bar/foo.png"]});
+        ->addList()
+        ->add({Document => ["bar/foo.png"]});
 
-   my $HashRef = $Manifest->manifest();
-   my $XMLString = $Manifest->getXML();
+    my $HashRef = $Manifest->manifest();
+    my $XMLString = $Manifest->getXML();
 
 =head1 METHODS
 
