@@ -5,13 +5,13 @@ use 5.12.0;
 use strict;
 use warnings;
 
-use Log::Log4perl ':easy';
-use WWW::SFDC::SessionManager;
+our $VERSION = '0.30'; # VERSION
 
+use Log::Log4perl ':easy';
 use Scalar::Util 'blessed';
 
 use Moo;
-with 'MooX::Singleton', 'WWW::SFDC::Role::Session', 'WWW::SFDC::Role::CRUD';
+with 'WWW::SFDC::Role::SessionConsumer', 'WWW::SFDC::Role::CRUD';
 
 
 has 'uri',
@@ -109,7 +109,7 @@ WWW::SFDC::Tooling - Interface to the Salesforce.com Tooling API
 
 =head1 VERSION
 
-version 0.25
+version 0.30
 
 =head1 SYNOPSIS
 
