@@ -5,6 +5,8 @@ use 5.12.0;
 use strict;
 use warnings;
 
+our $VERSION = '0.30'; # VERSION
+
 use Data::Dumper;
 use File::Path qw(mkpath);
 use File::Spec::Functions qw(splitpath);
@@ -85,8 +87,10 @@ sub makezip {
 
   $baseDir =~ s{(?<![/\\])$}{/};
 
+  chomp @files;
+
   @files = grep {-e $_ && !-d $_}
-    map {$baseDir.$_}
+    map {s/^(?!$baseDir)/$baseDir/r}
     @files;
 
   DEBUG "File list for zipping: " . Dumper \@files;
@@ -124,7 +128,7 @@ WWW::SFDC::Zip - Utilities for manipulating base64 encoded zip files.
 
 =head1 VERSION
 
-version 0.25
+version 0.30
 
 =head1 SYNOPSIS
 
